@@ -1,4 +1,4 @@
-`readFITSbintable` <-
+readFITSbintable <-
 function (zz, hdr)
 {
 ### Reader for FITS BINTABLE arrays
@@ -22,12 +22,20 @@ function (zz, hdr)
 ### Eliminate blanks in TTFORM etc. variables for new header parser
 ### AH 7/10/09
 ### Fixed apparent problem in padding calculation and read, 9/28/10 AH
+### Added "K" TFORMn; reads and records high and low 32-bit words separately,
+###   returning vector with double length.  Package int64 may be useful to
+###   reconstruct 64-bit word.  10/26/2012 AH
 ###
 ### Known incomplete:
 ###  * Data types X, C, M, P unimplemented (bit, complex, double complex,
 ###      array descriptor)
 ###
-  ## Determine array dimensions
+### Updated for new header handling, 12/30/12 AH
+
+    ## Parse header if full header is supplied instead of parsed version
+    if (nchar(hdr[1])==80) hdr <- parseHdr(hdr)
+
+    ## Determine array dimensions
     naxis1 <- as.numeric(hdr[which(hdr == "NAXIS1") + 1])   # bytes per row
     naxis2 <- as.numeric(hdr[which(hdr == "NAXIS2") + 1])   # rows
     tfields <- as.numeric(hdr[which(hdr == "TFIELDS") + 1]) # columns
@@ -79,6 +87,7 @@ function (zz, hdr)
             1])
         TDIMn[i] <- ifelse(length(tmp) != 1, "", tmp)
     }
+
     ## Work out formats and set up storage for each column.
     bsize <- integer(tfields)
     btype <- integer(tfields)
@@ -91,7 +100,7 @@ function (zz, hdr)
         tmp <- substr(TFORMn[i], 1, nc - 1)
         mult[i] <- ifelse(tmp == "", 1, as.numeric(tmp))
         ## Format; btype: 1 character, 2 logical, 3 integer,
-        ## 4 numeric, 5 complex
+        ## 4 numeric, 5 complex, 6 64-bit integer
         form <- tolower(substr(TFORMn[i], nc, nc))
         switch(form, l = {  # logical
             bsize[i] <- 1
@@ -109,48 +118,82 @@ function (zz, hdr)
             bsize[i] <- 4
             btype[i] <- 3
             bsign[i] <- TRUE
+        }, k = {            # 64-bit signed int (quadruple)
+            bsize[i] <- 8
+            btype[i] <- 6
+            bsign[i] <- TRUE
         }, a = {            # 8-bit character
             bsize[i] <- 1
             btype[i] <- 1
-            bsign[i] <- FALSE
         }, e = {            # 32-bit float (single)
             bsize[i] <- 4
             btype[i] <- 4
-            bsign[i] <- FALSE
         }, d = {            # 64-bit float (double)
             bsize[i] <- 8
             btype[i] <- 4
-            bsign[i] <- FALSE
-        }, stop("X, C, M, P not yet implemented \n"))
+        }, stop('Unknown TFORMn ', toupper(form),
+                ' (X, C, M, P not yet implemented)\n'))
+
         ## Set up storage arrays: rows = number of table rows, columns =
         ## multiplier (depth) of the cells in each column.  Characters are an
-        ## exception since they return strings.
-        if (btype[i] == 1) {
-            col[[i]] <- array("", dim = c(naxis2, 1))
-        }
-        else {
-            col[[i]] <- array(switch(btype[i], "", FALSE, NA,
-                NA, NA), dim = c(naxis2, mult[i]))
-        }
+        ## exception since they return strings.  64-bit integers need two
+        ## 32-bit words per value.
+        col[[i]] <- switch(btype[i],
+                           #1, char
+                           array("", dim = c(naxis2, 1)),
+                           #2, logical
+                           array(FALSE, dim = c(naxis2, mult[i])),
+                           #3, integer
+                           array(NA, dim = c(naxis2, mult[i])),
+                           #4, numeric
+                           array(NA, dim = c(naxis2, mult[i])),
+                           #5, complex
+                           array(NA, dim = c(naxis2, mult[i])),
+                           #6, 64-bit integer, 2 32-bit integers
+                           array(NA, dim = c(2*naxis2, mult[i]))
+                           )
     }
+
     ## Read data, row by row
     for (i in 1:naxis2) {
         for (j in 1:tfields) {
-            if (btype[j] <= 2) {  # character reads
-                col[[j]][i, ] <- readChar(zz, nchars = mult[j])
+            if (btype[j] <= 5) {
+                col[[j]][i, ] <- switch(btype[j],
+                                 # 1, char
+                                 readChar(zz, nchars = mult[j]),
+                                 # 2, logical
+                                 readChar(zz, nchars = mult[j]),
+                                 # 3, integer
+                                 readBin(zz, what = integer(), n = mult[j],
+                                         size = bsize[j], signed = bsign[j],
+                                         endian = "big"),
+                                 # 4, numeric
+                                 readBin(zz, what = numeric(), n = mult[j],
+                                         size = bsize[j],
+                                         endian = "big"),
+                                 # 5, complex
+                                 readBin(zz, what = complex(), n = mult[j],
+                                         size = bsize[j],
+                                         endian = "big")
+                   )
             }
-            else {
-                what <- switch(btype[j], character(), logical(),
-                  integer(), numeric(), complex())
-                col[[j]][i, ] <- readBin(zz, what = what, n = mult[j],
-                  size = bsize[j], signed = bsign[j], endian = "big")
+            if (btype[j] == 6) {
+                # 6, 64-bit signed integer, 2 32-bit integers
+                col[[j]][i, ] <- readBin(zz, what = integer(),
+                                    n = mult[j], size = 4,
+                                    endian = "big")
+                 col[[j]][i+naxis2, ] <- readBin(zz, what = integer(),
+                                    n = mult[j], size = 4,
+                                    endian = "big")
             }
         }
     }
+
     ## Finish reading block
     nbyte <- naxis1 * naxis2
     nbyte <- ifelse(nbyte%%2880 == 0, 0, 2880 - nbyte%%2880)
     tmp <- readBin(zz, 'raw', nbyte)
+
     ## Clean up before returning
     for (i in 1:tfields) {
         ## Apply scaling and offset where appropriate
@@ -163,15 +206,8 @@ function (zz, hdr)
             || btype[i] <= 2) {
             col[[i]] <- as.vector(col[[i]])
         }
-        ## Terminate character strings that end in ascii 0
-        if (btype[i] <= 2) {
-            lcol <- length(col[[i]])
-            txttmp <- character(lcol)
-            for (j in 1:lcol) txttmp[j] <-
-                strsplit((col[[i]][j]), 0)
-            col[[i]] <- unlist(txttmp)
-        }
     }
+
     ## Return data list: data for each column plus ancillary data.
     list(col = col, hdr = hdr, colNames = TTYPEn, colUnits = TUNITn,
         TNULLn = TNULLn, TSCALn = TSCALn, TZEROn = TZEROn, TDISPn = TDISPn)
